@@ -8,15 +8,16 @@ from gv_pubsub.pubsub import PubSub
 from gv_requests.requests import Requests
 from gv_utils.asyncio import check_event_loop
 from gv_utils.datetime import datetime
-from gv_utils.enums import MessageType, AttId
-from gv_utils.geometry import GeomType
+from gv_utils.enums import MessageKind, AttId
+from gv_utils.geometry import BaseGeometry
 from gv_utils.logger import Logger
 
 
 class Service:
-    samplings = {MessageType.karrusrd: 1 * 60, MessageType.metropme: 1 * 60, MessageType.tomtomfcd: 1 * 60}
+    samplings = {MessageKind.karrusrd: 1 * 60, MessageKind.metropme: 1 * 60, MessageKind.tomtomfcd: 1 * 60}
 
-    def __init__(self, logger: Logger, futures: list = None, callbacks: list = None):
+    def __init__(self, logger: Logger, futures: list = None, callbacks: dict = None,
+                 custom_data_decoder: Callable = None) -> None:
         if futures is None:
             futures = []
         if callbacks is None:
@@ -25,7 +26,7 @@ class Service:
         self.logger = logger
         self.futures = futures
         self.callbacks = callbacks
-        self.pubsub = None
+        self.pubsub = PubSub(self.logger, custom_data_decoder)
         self.requests = Requests(logger)
         self._mainfut = None
 
@@ -43,15 +44,15 @@ class Service:
 
     async def _run(self, redisaddr: str) -> None:
         try:
+            await self.pubsub.async_init(redisaddr)
             await self.async_init()
-            self.pubsub = PubSub(self.logger, redisaddr)
             self.logger.info('RPC client has started.')
             running = True
             while running:
                 try:
                     self._mainfut = asyncio.gather(
                         *self.futures,
-                        *[self._subscribe(datatype, callback) for datatype, callback in self.callbacks.items()]
+                        *[self._subscribe(datakind, callback) for datakind, callback in self.callbacks.items()]
                     )
                     await self._mainfut
                 except KeyboardInterrupt:
@@ -73,23 +74,23 @@ class Service:
             self._mainfut.cancel()
             self._mainfut = None
 
-    async def _publish(self, data: object, dataformat: str, datatimestamp, datatype: str) -> bool:
-        return await self.pubsub.publish(data, dataformat, datatimestamp, datatype)
+    async def _publish(self, data: object, dataformat: str, datatimestamp, datakind: str) -> bool:
+        return await self.pubsub.publish(data, dataformat, datatimestamp, datakind)
 
-    async def _subscribe(self, datatype: str, callback: Callable[[dict], Awaitable]) -> None:
-        await self.pubsub.subscribe(datatype, callback)
+    async def _subscribe(self, datakind: str, callback: Callable[[dict], Awaitable]) -> None:
+        await self.pubsub.subscribe(datakind, callback)
 
     async def _get_data_point(self, datapointeids: list = None, datatypeeids: list = None,
-                              area: GeomType = None) -> dict:
+                              area: BaseGeometry = None) -> dict:
         return await self.requests.get_data_point(datapointeids, datatypeeids, area)
 
-    async def _get_road(self, roadeids: list = None, area: GeomType = None) -> dict:
+    async def _get_road(self, roadeids: list = None, area: BaseGeometry = None) -> dict:
         return await self.requests.get_road(roadeids, area)
 
-    async def _get_zone_point(self, zoneeids: list = None, area: GeomType = None) -> dict:
+    async def _get_zone_point(self, zoneeids: list = None, area: BaseGeometry = None) -> dict:
         return await self.requests.get_zone_point(zoneeids, area)
 
-    async def _get_road_data_point(self, area: GeomType = None) -> dict:
+    async def _get_road_data_point(self, area: BaseGeometry = None) -> dict:
         return await self.requests.get_road_data_point(area)
 
     async def _get_data_point_data_quality(self, period: str, fromdatetime: datetime, datapointeids: list = None,
@@ -128,9 +129,9 @@ class Service:
     @staticmethod
     def _get_data_type_eid_from_data_points(datapoints: dict) -> str:
         datapointeid, datapoint = datapoints.popitem()
-        datatype = datapoint[AttId.datatypeeid]
+        datatypeeid = datapoint[AttId.datatypeeid]
         datapoints[datapointeid] = datapoint
-        return datatype
+        return datatypeeid
 
 
 def start(Application, threaded=False):
